//
//  Copyright: (C) 2005-2006 by Martin Koehler <mkoehler@users.berlios.de>
//  $Id: TreeComparator.h 28 2008-10-18 13:55:13Z martin $
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version
//

#import <Cocoa/Cocoa.h>
#import "Comparator.h"

@interface TreeComparator : Comparator
{
  SEL _parentSelector;   // selector to the parent node
  SEL _childrenSelector; // selector to an NSArray of children nodes
  NSArray *_childrenSortDescriptors; // if non-nil, children are sorted before comparison
  Comparator *_nodeComparator;  
}

#pragma mark Initializers
- (id)initWithParentSelector:(SEL)theParentSelector
            childrenSelector:(SEL)theChildrenSelector
      childrenSortDescriptor:(NSSortDescriptor *)theChildrenSortDescriptor
              nodeComparator:(Comparator *)theNodeComparator;

// calls above initizaler with sortdescriptor=nil
- (id)initWithParentSelector:(SEL)theParentSelector
            childrenSelector:(SEL)theChildrenSelector
              nodeComparator:(Comparator *)theNodeComparator;

// calls above initializer with @selector(parent) and @selector(children)
- (id)initWithChildrenSortDescriptor:(NSSortDescriptor *)theChildrenSortDescriptor
                      nodeComparator:(Comparator *)theNodeComparator;

// calls above initializer with @selector(parent) and @selector(children)
- (id)initWithNodeComparator:(Comparator *)theNodeComparator;

#pragma mark Getters
- (SEL)parentSelector;
- (SEL)childrenSelector;
- (NSSortDescriptor *)childrenSortDescriptor;
- (Comparator *)nodeComparator;

#pragma mark Setters
- (void)setParentSelector:(SEL)theParentSelector;
- (void)setChildrenSelector:(SEL)theChildrenSelector;
- (void)setChildrenSortDescriptor:(NSSortDescriptor *)theChildrenSortDescriptor;
- (void)setNodeComparator:(Comparator *)nodeComparator;

#pragma mark Comparators
- (BOOL)object:(id)obj1 
     hasEqualChildren:(id)obj2
     differenceDescription:(NSString **)difference;

- (BOOL)object:(id)obj1 
     isEqualToRegardlessOfChildren:(id)obj2;

// this is implemented by subclasses
- (BOOL)object:(id)obj1 
     isEqualToRegardlessOfChildren:(id)obj2
     differenceDescription:(NSString **)difference;

#pragma mark Reporting
- (NSString *)traceToNode:(id)node;
- (NSString *)augmentWithTraceDifferenceDescriptionBetween:(id)obj1 
                                                       and:(id)obj2
                                     differenceDescription:(NSString *)differenceDescription;
- (NSString *)describeDifferenceBetween:(id)obj1 
                                    and:(id)obj2
                                 reason:(NSString *)reason
                                 lvalue:(id)lvalue
                                 rvalue:(id)rvalue;
@end
