//
//  Copyright: (C) 2006-2007 by Martin Koehler <mkoehler@users.berlios.de>
//  $Id: MacrosAndGlobals.h 21 2008-01-31 12:53:43Z martin $
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version
//

#import <Cocoa/Cocoa.h>

static inline NSPoint CGPointToNSPoint(CGPoint point)
{
  return *((NSPoint *)&point);
}

static inline CGPoint NSPointToCGPoint(NSPoint point)
{
  return *((CGPoint *)&point);
}

static inline NSRect CGRectToNSRect(CGRect rect)
{
  return *((NSRect *)&rect);
}

static inline CGRect NSRectToCGRect(NSRect rect)
{
  return *((CGRect *)&rect);
}

// useful macros
#define NSLOG_THAT_METHOD_CALL \
  NSLog(@"- [%@ %s] called", \
        [self className], _cmd)

// see http://docs.freebsd.org/info/gcc/gcc.info.Macro_Varargs.html

#define NSLOG_THAT_METHOD_CALL_AND_MESSAGE(format, args...) \
  NSLog(@"- [%@ %s]: %@", \
        [self className], _cmd, \
        [NSString stringWithFormat: format , ## args ] )


#define NSLOG_THAT_METHOD_IS_ABSTRACT \
  NSLog(@"- [%@ %s] is 'abstract' and should be implemented by subclasses", \
        [self className], _cmd)


#define FOREACH(type, object, collection) FOREACHE(type,object,collection,object ## enumerator)
#define FOREACHE(type,object,collection,enumerator) \
  NSEnumerator *enumerator = [collection objectEnumerator]; \
  type object; \
  IMP next ## object ## in ## enumerator = [enumerator methodForSelector:@selector(nextObject)]; \
  while((object = next ## object ## in ## enumerator(enumerator,@selector(nextObject))))

#define REVERSE_FOREACH(type, object, collection) REVERSE_FOREACHE(type,object,collection,object ## enumerator)
#define REVERSE_FOREACHE(type,object,collection,enumerator) \
  NSEnumerator *enumerator = [collection reverseObjectEnumerator]; \
  type object; \
  IMP next ## object ## in ## enumerator = [enumerator methodForSelector:@selector(nextObject)]; \
  while((object = next ## object ## in ## enumerator(enumerator,@selector(nextObject))))
