//
//  Copyright: (C) 2006-2008 by Martin Koehler <mkoehler@users.berlios.de>
//  $Id: CdrdaoFrontend(ReadCD).h 96 2008-06-23 18:50:28Z martin $
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version
//

#import <Cocoa/Cocoa.h>
#import "CdrdaoFrontend.h"

// keys for read-cd operation
extern NSString *CdrdaoReadCd_ModeKey;
extern NSString *CdrdaoReadCd_DiscDurationInMSFKey;
extern NSString *CdrdaoReadCd_DiscDurationInFramesKey;
extern NSString *CdrdaoReadCd_DiscDurationInRawBytesKey; // complete 2352 frames
extern NSString *CdrdaoReadCd_FlagsKey;

@interface CdrdaoFrontend (ReadCD)

#pragma mark Reading CD

+ (NSTask *)readCdFromDRDevice:(DRDevice *)device
                 sessionNumber:(unsigned)sessionNumber
                   shallRipRaw:(BOOL)shallRipRaw
          errorCorrectionLevel:(unsigned)errorCorrectionLevel
                       tocPath:(NSString *)tocPath
                       binPath:(NSString *)binPath
                      delegate:(id <DiscOperationProgressDelegate>)delegate;

#pragma mark Reading CD (synchronous), not terminatable

// blocks until finished
// progress is reported via delegate when non-nil
// note: tocPath and binPath must have extensions "toc" and "bin"
+ (NSDictionary *)synchReadCdFromDRDevice:(DRDevice *)device
                            sessionNumber:(unsigned)sessionNumber
                              shallRipRaw:(BOOL)shallRipRaw
                     errorCorrectionLevel:(unsigned)errorCorrectionLevel
                                  tocPath:(NSString *)tocPath
                                  binPath:(NSString *)binPath
                                 delegate:(id <DiscOperationProgressDelegate>)delegate;

#pragma mark Internal Helpers for launching read-cd

// note: tocPath and binPath must have extensions "toc" and "bin"
+ (NSTask *)readCdFromScsiLibDevice:(NSString *)device
                      sessionNumber:(unsigned)sessionNumber
                        shallRipRaw:(BOOL)shallRipRaw
               errorCorrectionLevel:(unsigned)errorCorrectionLevel
                            tocPath:(NSString *)tocPath
                            binPath:(NSString *)binPath
                     standardOutput:(NSPipe *)pipe;

// device is unlocked, and the above method is called with the corresponding scsilib devive name
// note: tocPath and binPath must have extensions "toc" and "bin"
+ (NSTask *)readCdFromDRDevice:(DRDevice *)device
                 sessionNumber:(unsigned)sessionNumber
                   shallRipRaw:(BOOL)shallRipRaw
          errorCorrectionLevel:(unsigned)errorCorrectionLevel
                       tocPath:(NSString *)tocPath
                       binPath:(NSString *)binPath
                standardOutput:(NSPipe *)pipe;

#pragma mark Internal Helper for interpreting output from read-cd

// threaded wrapper around below method
+ (void)threadedProcessReadCdStandardOutput:(NSArray *)arguments;

+ (NSMutableDictionary *)processReadCdStandardOutput:(NSPipe *)pipe
                                       sessionNumber:(unsigned)sessionNumber
                                             binPath:(NSString *)binPath
                                            delegate:(id <DiscOperationProgressDelegate>)delegate;

+ (DiscOperationState)scanReadCdOutputLine:(NSString *)outputLine
                      augmentDictionary:(NSMutableDictionary *)dictionary;

+ (NSString *)scanForErrorInReadCdOutputLine:(NSString *)outputLine;

@end
