//
//  Copyright: (C) 2006-2008 by Martin Koehler <mkoehler@users.berlios.de>
//  $Id: DiscOperation.h 132 2009-09-05 12:53:41Z martin $
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version
//

#import <Cocoa/Cocoa.h>
#import "DiscOperationStatus.h"
#import <DiscRecording/DiscRecording.h>

// dictionary keys
extern NSString *DiscOperation_RealTaskDurationKey;
extern NSString *DiscOperation_ReturnCodeKey;
extern NSString *DiscOperation_ErrorKey;

// interface for NSCondition (public since 10.5, but was present+private <10.5)
//#ifndef MAC_OS_X_VERSION_10_5
@class NSCondition;
@interface NSCondition : NSObject <NSLocking>
- (void)wait;
- (void)broadcast;
// only since leopard...
//- (void)setName:(NSString *)name;
@end
//#endif

@interface DiscOperation : NSObject <DiscOperationProgressDelegate>
{
  DRDevice *_device;
  NSString *_mediumClass;

  id <DiscOperationProgressDelegate> _delegate;
  
  NSCondition *_preCommandHasFinishedCondition;
  BOOL _shallContinueAfterPreCommand;
}

#pragma mark Initializers

- (id)initWithDRDevice:(DRDevice *)theDevice
              delegate:(id <DiscOperationProgressDelegate>)theDelegate;

#pragma mark Starting and Stopping the operation

- (void)start;
- (void)stop;

#pragma mark Internal (unlocking and internal disc operation)

- (void)launchUnlockPreCommand;
- (void)launchOperationCommand; // template method
- (void)threadedLaunchUnlockPreCommand;

#pragma mark Internal class methods

+ (void)launchUnlockPreCommand:(DRDevice *)device
                      delegate:(id <DiscOperationProgressDelegate>)delegate;

+ (BOOL)scanOutputLine:(NSString *)outputLine
     augmentDictionary:(NSMutableDictionary *)dictionary
    outStateIndication:(DiscOperationState *)stateIndication;

@end
