//
//  Copyright: (C) 2005-2008 by Martin Koehler <mkoehler@users.berlios.de>
//  $Id: ImageFile.h 111 2008-10-26 18:26:45Z martin $
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version
//

#import <Cocoa/Cocoa.h>

@class DRTrack;

@interface ImageFileHelper : NSObject
{
  NSString *_path;
  BOOL _attached;

  NSMutableDictionary *_attachHdiUtilPlist;

  BOOL _exists, _mountable;
  uint64_t _imageFileSize, _totalSize;
  NSString *_imageFileSizeAsString, *_totalSizeAsString;
  NSString *_formatDescriptionString;

  NSMutableArray *_partitions; // contains NSMutableDictionaries, each with keys 
                              // SBPartitionInfo_NameKey, SBPartitionInfo_FileSystemTypeKey

  BOOL _containsRawTracks;
}

#pragma mark Class Methodds

// returns autoreleased instance
+ (ImageFileHelper *)imageFileHelperFromPath:(NSString *)path;

+ (BOOL)pathIsValidImage:(NSString *)path;

#pragma mark Initializer

- (id)initFromPath:(NSString *)path;

#pragma mark Getters

- (NSString *)path;

- (BOOL)exists;
- (BOOL)mountable;
- (uint64_t)imageFileSize;
- (uint64_t)totalSize;
- (NSString *)imageFileSizeAsString;
- (NSString *)totalSizeAsString;
- (NSString *)formatDescriptionString;

- (BOOL)containsRawTracks;
- (NSMutableArray *)partitions;

// returns true iff the image is attached ("mounted")
- (BOOL)attached;

//
// if the image is attached, 
//  this returns the autoreleased list of mountpoints (NSStrings, f.ex. "/Volumes/foo")
// else, returns nil
//
- (NSArray *)mountPoints;

//
// if the image is attached, 
//  this returns the list of the BSD device names (NSStrings, f.ex. "/dev/disk3s2")
// else, returns nil
//
- (NSArray *)devices;

#pragma mark Setters

- (void)setPath:(NSString *)thePath;

- (void)setImageFileSizeAsString:(NSString *)theImageFileSizeAsString;
- (void)setTotalSizeAsString:(NSString *)theTotalSizeAsString;
- (void)setFormatDescriptionString:(NSString *)theFormatDescriptionString;

- (void)setPartitions:(NSMutableArray *)thePartitions;

- (void)setAttached:(BOOL)isAttached;
- (void)setAttachHdiUtilPlist:(NSMutableDictionary *)plist;

#pragma mark (Un)mounting Image

- (BOOL)attach;
- (BOOL)detach;

#pragma mark Internal

- (void)updateWithNewPath;

- (void)extractPlistInfoAboutMountPoints:(NSMutableArray *)mountPoints
                                 devices:(NSMutableArray *)devices;

// track array is autoreleased !
- (void)getImageInfoViaDRBurnAboutDiscUsage:(uint64_t *)outDiscUsage
                             DRTrackArray:(NSMutableArray **)outTrackArray
                        containsRawTracks:(BOOL *)outContainsRawTracks;

- (void)addTrackInfo:(DRTrack *)track
         toDiscUsage:(uint64_t *)outDiscUsage
        DRTrackArray:(NSMutableArray **)outTrackArray
               isRaw:(BOOL *)outIsRaw;

@end
